/**
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef LEETM_BARRIER_H_
#define LEETM_BARRIER_H_

#include <unistd.h>
#include <pthread.h>

#if defined(_POSIX_BARRIERS) && (_POSIX_BARRIERS - 20012L) >= 0

class Barrier {
	public:
		void Init(unsigned thread_count) {
			pthread_barrier_init(&barrier, NULL, thread_count);
		}

		void Wait() {
			pthread_barrier_wait(&barrier);
		}

	private:
		pthread_barrier_t barrier;
};

#else

class Barrier {
	public:
		void Init(unsigned thread_count) {
			total_count = thread_count;
			current_count = 0;
			pthread_mutex_init(&mutex, NULL);
			pthread_cond_init(&cond, NULL);
		}

		void Wait() {
			pthread_mutex_lock(&mutex);

			if(++current_count == total_count) {
				pthread_cond_broadcast(&cond);
			} else {
				pthread_cond_wait(&cond, &mutex);
			}

			pthread_mutex_unlock(&mutex);
		}

	private:
		unsigned total_count;
		unsigned current_count;
		pthread_mutex_t mutex;
		pthread_cond_t cond;
};

#endif /* POSIX */


#endif /* LEETM_BARRIER_H_ */
